@@ -21,17 +21,20 @@
 
 #include <drivers/atmodem/atutil.h>
 
-extern void ifx_voicecall_init();
-extern void ifx_voicecall_exit();
+extern void ifx_voicecall_init(void);
+extern void ifx_voicecall_exit(void);
 
-extern void ifx_audio_settings_init();
-extern void ifx_audio_settings_exit();
+extern void ifx_audio_settings_init(void);
+extern void ifx_audio_settings_exit(void);
 
-extern void ifx_radio_settings_init();
-extern void ifx_radio_settings_exit();
+extern void ifx_radio_settings_init(void);
+extern void ifx_radio_settings_exit(void);
 
-extern void ifx_gprs_context_init();
-extern void ifx_gprs_context_exit();
+extern void ifx_gprs_context_init(void);
+extern void ifx_gprs_context_exit(void);
 
-extern void ifx_stk_init();
-extern void ifx_stk_exit();
+extern void ifx_stk_init(void);
+extern void ifx_stk_exit(void);
+
+extern void ifx_ctm_init(void);
+extern void ifx_ctm_exit(void);
\ No newline at end of file
