@@ -27,8 +27,10 @@ extern "C" {
 #endif
 
 #define PN_PHONE_INFO				0x1B
-#define PN_EPOC_INFO    			98
-#define INFO_TIMEOUT 				5
+#define PN_MODEM_INFO				0xC5
+
+#define PN_EPOC_INFO				98
+#define INFO_TIMEOUT				5
 
 enum info_isi_cause {
 	INFO_OK =				0x00,
@@ -50,6 +52,7 @@ enum info_message_id {
 };
 
 enum info_subblock {
+	INFO_SB_MODEMSW_VERSION =		0x00,
 	INFO_SB_PRODUCT_INFO_NAME =		0x01,
 	INFO_SB_PRODUCT_INFO_MANUFACTURER =	0x07,
 	INFO_SB_SN_IMEI_PLAIN =			0x41,
@@ -72,7 +75,7 @@ enum info_version_type {
 };
 
 enum info_pp_feature {
-	INFO_PP_MAX_PDP_CONTEXTS = 0xCA
+	INFO_PP_MAX_PDP_CONTEXTS =		0xCA
 };
 
 #ifdef __cplusplus
