@@ -52,6 +52,7 @@
 #define DEFAULT_SOCK_PATH "./server_sock"
 #define IFCONFIG_PATH "/sbin/ifconfig"
 
+static gboolean data_mode = FALSE;
 static int modem_mode = 0;
 static int modem_creg = 0;
 static int modem_cgreg = 0;
@@ -147,6 +148,7 @@ static void ppp_disconnect(GAtPPPDisconnectReason reason, gpointer user)
 	g_at_server_set_debug(server, server_debug, "Server");
 
 	g_at_server_send_final(server, G_AT_SERVER_RESULT_NO_CARRIER);
+	data_mode = FALSE;
 }
 
 static gboolean update_ppp(gpointer user)
@@ -159,16 +161,11 @@ static gboolean update_ppp(gpointer user)
 	return FALSE;
 }
 
-static gboolean setup_ppp(gpointer user)
+static void setup_ppp(gpointer user)
 {
 	GAtServer *server = user;
 	GAtIO *io;
 
-	if (getuid() != 0) {
-		g_print("Need root privilege for PPP connection\n");
-		return FALSE;
-	}
-
 	io = g_at_server_get_io(server);
 
 	g_at_server_suspend(server);
@@ -177,7 +174,7 @@ static gboolean setup_ppp(gpointer user)
 	ppp = g_at_ppp_server_new_from_io(io, "192.168.1.1");
 	if (ppp == NULL) {
 		g_at_server_resume(server);
-		return FALSE;
+		return;
 	}
 
 	g_at_ppp_set_debug(ppp, server_debug, "PPP");
@@ -189,8 +186,6 @@ static gboolean setup_ppp(gpointer user)
 	g_at_ppp_set_disconnect_function(ppp, ppp_disconnect, server);
 
 	g_idle_add(update_ppp, ppp);
-
-	return FALSE;
 }
 
 static void cgmi_cb(GAtServer *server, GAtServerRequestType type,
@@ -348,6 +343,9 @@ static gboolean do_netreg(gpointer user)
 	GAtServer *server = user;
 	char buf[32];
 
+	if (data_mode)
+		return FALSE;
+
 	network_status = 1;
 
 	switch (modem_creg) {
@@ -558,6 +556,7 @@ error:
 static void cgdata_cb(GAtServer *server, GAtServerRequestType type,
 			GAtResult *cmd, gpointer user)
 {
+	GAtIO *io;
 	if (modem_mode == 0) {
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
 		return;
@@ -572,7 +571,10 @@ static void cgdata_cb(GAtServer *server, GAtServerRequestType type,
 		break;
 	case G_AT_SERVER_REQUEST_TYPE_SET:
 		g_at_server_send_intermediate(server, "CONNECT");
-		g_idle_add(setup_ppp, server);
+		data_mode = TRUE;
+
+		io = g_at_server_get_io(server);
+		g_at_io_set_write_done(io, setup_ppp, server);
 		break;
 	default:
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
@@ -793,8 +795,11 @@ static void dial_cb(GAtServer *server, GAtServerRequestType type,
 
 	c = *dial_str;
 	if (c == '*' || c == '#' || c == 'T' || c == 't') {
+		GAtIO *io = g_at_server_get_io(server);
+
 		g_at_server_send_intermediate(server, "CONNECT");
-		g_idle_add(setup_ppp, server);
+		data_mode = TRUE;
+		g_at_io_set_write_done(io, setup_ppp, server);
 	}
 
 	return;
